(ns cipher-analytical-machine.analyzers.simple-substitution
  (:require [clojure.string :as cs]
            [clojure.math.combinatorics :as comb]
            [cipher-analytical-machine.ciphers.simple-substitution :as ss]
            [cipher-analytical-machine.analyzers.analyzers :as analyzers]
            [cipher-analytical-machine.analyzers.caesar :as caesar])
  (:gen-class))

(defn get-possible-key-vector
  "Conver a possible key map to a vector of possible keys. For example, if you have a map: {\\a [\\a \\b \\c] \\b [\\b \\c] \\c [\\a \\c]}, then the vector is [[\\a \\b \\c] [\\b \\c] [\\a \\c]]."
  [possible-key-map symbols]
  (reduce
   (fn [acc el]
     (conj acc (get possible-key-map el)))
   [] symbols))

(defn get-possible-keys
  "Return possible keys for a possible key vector. For example, if you have a map: {\\a [\\a \\b \\c] \\b [\\b \\c] \\c [\\a \\c]}, then the vector is [[\\a \\b \\c] [\\b \\c] [\\a \\c]] and keys (\"aba\" \"abc\" ...)."
  [possible-key-vector]
  (->> (apply comb/cartesian-product possible-key-vector)
       (map cs/join)))

